data Nat : Set 
{ zero : Nat
; suc  : Nat -> Nat
}

data Bool : Set
{ true  : Bool
; false : Bool 
}

fun leq : Nat -> Nat -> Bool
{ leq zero n = true
; leq (suc m) zero = false
; leq (suc m) (suc n) = leq m n
}

data Id (A : Set) (a : A) : A -> Set
{ refl : Id A a a
}

fun True : Bool -> Set
{ True b = Id Bool b true
}
let triv : True true
         = refl Bool true

fun False : Bool -> Set
{ False b = Id Bool b false
}
let triv' : False false
          = refl Bool false

fun leFalse : (n : Nat) -> (m : Nat) -> False (leq n m) -> True (leq m n)
{ leFalse  n       zero   p = triv
; leFalse (suc n) (suc m) p = leFalse n m p
-- ; leFalse zero (suc m) () -- IMPOSSIBLE
}

data SList : Nat -> Set
{ snil  : SList zero
; scons : (m : Nat) -> (n : Nat) -> True (leq n m)  -> SList n -> SList m
} 

fun max : Nat -> Nat -> Nat
{ max n m = case leq n m 
  { true -> m
  ; false -> n
  }
}

fun maxLemma : (n : Nat) -> (m : Nat) -> (k : Nat) ->
              True (leq n k) -> True (leq m k) -> True (leq (max n m) k)
{ maxLemma n m k p q = case leq n m 
  { true  -> q
  ; false -> p
  } 
}

fun insert : (m : Nat) -> (n : Nat) -> SList n -> SList (max n m)
{ insert m .zero snil = scons m zero triv snil
; insert m .n (scons n k p l) = case leq n m 
  { true  -> scons m n triv (scons n k p l)
  ; false -> scons n (max k m) (maxLemma k m n p (leFalse n m triv')) 
                   (insert m k l)
  }
}